@@ -21,7 +21,9 @@ from django.test import TestCase
 from django.utils import six
 
 from rest_framework import serializers
-from rest_framework.compat import unicode_repr
+from rest_framework.compat import postgres_fields, unicode_repr
+
+from .models import NestedForeignKeySource
 
 
 def dedent(blocktext):
@@ -379,6 +381,54 @@ class TestGenericIPAddressFieldValidation(TestCase):
                          '{0}'.format(s.errors))
 
 
+@pytest.mark.skipUnless(postgres_fields, 'postgres is required')
+class TestPosgresFieldsMapping(TestCase):
+    def test_hstore_field(self):
+        class HStoreFieldModel(models.Model):
+            hstore_field = postgres_fields.HStoreField()
+
+        class TestSerializer(serializers.ModelSerializer):
+            class Meta:
+                model = HStoreFieldModel
+                fields = ['hstore_field']
+
+        expected = dedent("""
+            TestSerializer():
+                hstore_field = HStoreField()
+        """)
+        self.assertEqual(unicode_repr(TestSerializer()), expected)
+
+    def test_array_field(self):
+        class ArrayFieldModel(models.Model):
+            array_field = postgres_fields.ArrayField(base_field=models.CharField())
+
+        class TestSerializer(serializers.ModelSerializer):
+            class Meta:
+                model = ArrayFieldModel
+                fields = ['array_field']
+
+        expected = dedent("""
+            TestSerializer():
+                array_field = ListField(child=CharField(label='Array field', validators=[<django.core.validators.MaxLengthValidator object>]))
+        """)
+        self.assertEqual(unicode_repr(TestSerializer()), expected)
+
+    def test_json_field(self):
+        class JSONFieldModel(models.Model):
+            json_field = postgres_fields.JSONField()
+
+        class TestSerializer(serializers.ModelSerializer):
+            class Meta:
+                model = JSONFieldModel
+                fields = ['json_field']
+
+        expected = dedent("""
+            TestSerializer():
+                json_field = JSONField(style={'base_template': 'textarea.html'})
+        """)
+        self.assertEqual(unicode_repr(TestSerializer()), expected)
+
+
 # Tests for relational field mappings.
 # ------------------------------------
 
@@ -1116,6 +1166,25 @@ class Test5004UniqueChoiceField(TestCase):
 
 
 class TestFieldSource(TestCase):
+    def test_traverse_nullable_fk(self):
+        """
+        A dotted source with nullable elements uses default when any item in the chain is None. #5849.
+
+        Similar to model example from test_serializer.py `test_default_for_multiple_dotted_source` method,
+        but using RelatedField, rather than CharField.
+        """
+        class TestSerializer(serializers.ModelSerializer):
+            target = serializers.PrimaryKeyRelatedField(
+                source='target.target', read_only=True, allow_null=True, default=None
+            )
+
+            class Meta:
+                model = NestedForeignKeySource
+                fields = ('target', )
+
+        model = NestedForeignKeySource.objects.create()
+        assert TestSerializer(model).data['target'] is None
+
     def test_named_field_source(self):
         class TestSerializer(serializers.ModelSerializer):
 
