@@ -10,9 +10,7 @@ from rest_framework import (
     filters, generics, pagination, permissions, serializers
 )
 from rest_framework.compat import coreapi, coreschema, get_regex_pattern, path
-from rest_framework.decorators import (
-    api_view, detail_route, list_route, schema
-)
+from rest_framework.decorators import action, api_view, schema
 from rest_framework.request import Request
 from rest_framework.routers import DefaultRouter, SimpleRouter
 from rest_framework.schemas import (
@@ -25,7 +23,7 @@ from rest_framework.utils import formatting
 from rest_framework.views import APIView
 from rest_framework.viewsets import GenericViewSet, ModelViewSet
 
-from .models import BasicModel
+from .models import BasicModel, ForeignKeySource
 
 factory = APIRequestFactory()
 
@@ -51,6 +49,10 @@ class ExampleSerializer(serializers.Serializer):
     hidden = serializers.HiddenField(default='hello')
 
 
+class AnotherSerializerWithDictField(serializers.Serializer):
+    a = serializers.DictField()
+
+
 class AnotherSerializerWithListFields(serializers.Serializer):
     a = serializers.ListField(child=serializers.IntegerField())
     b = serializers.ListSerializer(child=serializers.CharField())
@@ -67,25 +69,32 @@ class ExampleViewSet(ModelViewSet):
     filter_backends = [filters.OrderingFilter]
     serializer_class = ExampleSerializer
 
-    @detail_route(methods=['post'], serializer_class=AnotherSerializer)
+    @action(methods=['post'], detail=True, serializer_class=AnotherSerializer)
     def custom_action(self, request, pk):
         """
         A description of custom action.
         """
         return super(ExampleSerializer, self).retrieve(self, request)
 
-    @detail_route(methods=['post'], serializer_class=AnotherSerializerWithListFields)
+    @action(methods=['post'], detail=True, serializer_class=AnotherSerializerWithDictField)
+    def custom_action_with_dict_field(self, request, pk):
+        """
+        A custom action using a dict field in the serializer.
+        """
+        return super(ExampleSerializer, self).retrieve(self, request)
+
+    @action(methods=['post'], detail=True, serializer_class=AnotherSerializerWithListFields)
     def custom_action_with_list_fields(self, request, pk):
         """
         A custom action using both list field and list serializer in the serializer.
         """
         return super(ExampleSerializer, self).retrieve(self, request)
 
-    @list_route()
+    @action(detail=False)
     def custom_list_action(self, request):
         return super(ExampleViewSet, self).list(self, request)
 
-    @list_route(methods=['post', 'get'], serializer_class=EmptySerializer)
+    @action(methods=['post', 'get'], detail=False, serializer_class=EmptySerializer)
     def custom_list_action_multiple_methods(self, request):
         return super(ExampleViewSet, self).list(self, request)
 
@@ -200,6 +209,16 @@ class TestRouterGeneratedSchema(TestCase):
                             coreapi.Field('d', required=False, location='form', schema=coreschema.String(title='D')),
                         ]
                     ),
+                    'custom_action_with_dict_field': coreapi.Link(
+                        url='/example/{id}/custom_action_with_dict_field/',
+                        action='post',
+                        encoding='application/json',
+                        description='A custom action using a dict field in the serializer.',
+                        fields=[
+                            coreapi.Field('id', required=True, location='path', schema=coreschema.String()),
+                            coreapi.Field('a', required=True, location='form', schema=coreschema.Object(title='A')),
+                        ]
+                    ),
                     'custom_action_with_list_fields': coreapi.Link(
                         url='/example/{id}/custom_action_with_list_fields/',
                         action='post',
@@ -558,6 +577,51 @@ class TestSchemaGeneratorWithRestrictedViewSets(TestCase):
         assert schema == expected
 
 
+class ForeignKeySourceSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = ForeignKeySource
+        fields = ('id', 'name', 'target')
+
+
+class ForeignKeySourceView(generics.CreateAPIView):
+    queryset = ForeignKeySource.objects.all()
+    serializer_class = ForeignKeySourceSerializer
+
+
+@unittest.skipUnless(coreapi, 'coreapi is not installed')
+class TestSchemaGeneratorWithForeignKey(TestCase):
+    def setUp(self):
+        self.patterns = [
+            url(r'^example/?$', ForeignKeySourceView.as_view()),
+        ]
+
+    def test_schema_for_regular_views(self):
+        """
+        Ensure that AutoField foreign keys are output as Integer.
+        """
+        generator = SchemaGenerator(title='Example API', patterns=self.patterns)
+        schema = generator.get_schema()
+
+        expected = coreapi.Document(
+            url='',
+            title='Example API',
+            content={
+                'example': {
+                    'create': coreapi.Link(
+                        url='/example/',
+                        action='post',
+                        encoding='application/json',
+                        fields=[
+                            coreapi.Field('name', required=True, location='form', schema=coreschema.String(title='Name')),
+                            coreapi.Field('target', required=True, location='form', schema=coreschema.Integer(description='Target', title='Target')),
+                        ]
+                    )
+                }
+            }
+        )
+        assert schema == expected
+
+
 @unittest.skipUnless(coreapi, 'coreapi is not installed')
 class Test4605Regression(TestCase):
     def test_4605_regression(self):
@@ -807,15 +871,15 @@ class SchemaGenerationExclusionTests(TestCase):
         assert should_include == expected
 
     def test_deprecations(self):
-        with pytest.warns(PendingDeprecationWarning) as record:
+        with pytest.warns(DeprecationWarning) as record:
             @api_view(["GET"], exclude_from_schema=True)
             def view(request):
                 pass
 
         assert len(record) == 1
         assert str(record[0].message) == (
-            "The `exclude_from_schema` argument to `api_view` is pending "
-            "deprecation. Use the `schema` decorator instead, passing `None`."
+            "The `exclude_from_schema` argument to `api_view` is deprecated. "
+            "Use the `schema` decorator instead, passing `None`."
         )
 
         class OldFashionedExcludedView(APIView):
@@ -829,13 +893,13 @@ class SchemaGenerationExclusionTests(TestCase):
         ]
 
         inspector = EndpointEnumerator(patterns)
-        with pytest.warns(PendingDeprecationWarning) as record:
+        with pytest.warns(DeprecationWarning) as record:
             inspector.get_api_endpoints()
 
         assert len(record) == 1
         assert str(record[0].message) == (
             "The `OldFashionedExcludedView.exclude_from_schema` attribute is "
-            "pending deprecation. Set `schema = None` instead."
+            "deprecated. Set `schema = None` instead."
         )
 
 
@@ -865,11 +929,11 @@ class NamingCollisionViewSet(GenericViewSet):
     """
     permision_class = ()
 
-    @list_route()
+    @action(detail=False)
     def detail(self, request):
         return {}
 
-    @list_route(url_path='detail/export')
+    @action(detail=False, url_path='detail/export')
     def detail_export(self, request):
         return {}
 
@@ -949,7 +1013,10 @@ class TestURLNamingCollisions(TestCase):
 
         generator = SchemaGenerator(title='Naming Colisions', patterns=patterns)
         schema = generator.get_schema()
-        desc = schema['detail_0'].description  # not important here
+
+        # not important here
+        desc_0 = schema['detail']['detail_export'].description
+        desc_1 = schema['detail_0'].description
 
         expected = coreapi.Document(
             url='',
@@ -959,12 +1026,12 @@ class TestURLNamingCollisions(TestCase):
                     'detail_export': coreapi.Link(
                         url='/from-routercollision/detail/export/',
                         action='get',
-                        description=desc)
+                        description=desc_0)
                 },
                 'detail_0': coreapi.Link(
                     url='/from-routercollision/detail/',
                     action='get',
-                    description=desc
+                    description=desc_1
                 )
             }
         )
@@ -1046,7 +1113,7 @@ def test_head_and_options_methods_are_excluded():
 
     class AViewSet(ModelViewSet):
 
-        @detail_route(methods=['options', 'get'])
+        @action(methods=['options', 'get'], detail=True)
         def custom_action(self, request, pk):
             pass
 
